// constants

const float PI       = 3.141592;
const float TWO_PI   = PI * 2.0;
const float HALF_PI  = PI * 0.5;
const float DEG2RAD  = PI / 180.0;
const float EPS      = 0.0000001;

float saturate(float c) { return clamp(c,0.0,1.0); }
vec2 saturate(vec2 c) { return clamp(c,vec2(0.0),vec2(1.0)); }
vec3 saturate(vec3 c) { return clamp(c,vec3(0.0),vec3(1.0)); }
vec4 saturate(vec4 c) { return clamp(c,vec4(0.0),vec4(1.0)); }
float rangeValue(float t, float a, float b) { return saturate((t-a)/(b-a)); }
vec2 rangeValue(vec2 t, vec2 a, vec2 b) { return saturate((t-a)/(b-a)); }
float scaleAround(float x, float p, float s) { return (x - p) * s + p; }
vec2 scaleAround(vec2 x, vec2 p, float s) { return (x - p) * s + p; }
vec2 scaleAround(vec2 x, vec2 p, vec2 s) { return (x - p) * s + p; }
vec3 scaleAround(vec3 x, vec3 p, vec3 s) { return (x - p) * s + p; }

float modfi2(float a,float b) {
    return floor(mod(floor(a),floor(b)));
}

// angles

vec2 rotateZ(vec2 v, float a) {
    vec2 sc = vec2(sin(a),cos(a));
    vec2 ret = v;
    ret.x = v.x * sc.y - v.y * sc.x;
    ret.y = v.x * sc.x + v.y * sc.y;
    return ret;
}
vec2 rotateZ(vec2 v, vec2 pivot, float a) {
    vec2 sc = vec2(sin(a),cos(a));
    v -= pivot;
    vec2 ret = v;
    ret.x = v.x * sc.y - v.y * sc.x;
    ret.y = v.x * sc.x + v.y * sc.y;
    return ret + pivot;
}

// hashes

float hash11(float x) {
    return fract(sin(x) * 43758.5453);
}
float hash12(vec2 p) {
    vec3 p3  = fract(vec3(p.xyx) * 443.8975);
    p3 += dot(p3, p3.yzx + 19.19);
    return fract((p3.x + p3.y) * p3.z);
}
float hash13(vec3 p) {
    vec3 p3  = fract(vec3(p) * 443.8975);
    p3 += dot(p3, p3.yzx + 19.19);
    return fract((p3.x + p3.y) * p3.z);
}
vec2 hash21(float x) {
    return fract(sin(x) * vec2(43758.5453,5928.4124));
}
vec2 hash22(vec2 p) {
    vec3 p3 = fract(vec3(p.xyx) * 443.8975);
    p3 += dot(p3, p3.yzx+19.19);
    return fract(vec2((p3.x + p3.y)*p3.z, (p3.x+p3.z)*p3.y));
}
vec3 hash31(float p) {
   vec3 p3 = fract(vec3(p) * vec3(.1031, .1030, .0973));
   p3 += dot(p3, p3.yzx+19.19);
   return fract((p3.xxy+p3.yzz)*p3.zyx);
}
vec3 hash32(vec2 p) {
   vec3 p3 = fract(p.xyx * vec3(.1031, .1030, .0973));
   p3 += dot(p3, p3.yzx+19.19);
   return fract((p3.xxy+p3.yzz)*p3.zyx);
}
vec3 hash33(vec3 p) {
   vec3 p3 = fract(p.xyz * vec3(.1031, .1030, .0973));
   p3 += dot(p3, p3.yzx+19.19);
   return fract((p3.xxy+p3.yzz)*p3.zyx);
}

float rand11(float x) {
    return hash11(x) * 2.0 - 1.0;
}
float rand12(vec2 p) {
    return hash12(p) * 2.0 - 1.0;
}
vec2 rand21(float x) {
    return hash21(x) * 2.0 - 1.0;
}
vec2 rand22(vec2 p) {
    return hash22(p) * 2.0 - 1.0;
}
vec3 rand31(float p) {
    return hash31(p) * 2.0 - 1.0;
}