/*
 * Particle routines
 *
 * 8 floats per particle:
 * [pos.xy, vel.xy]
 * [color.g, color.rb, scale, timeout]
 */

float getIndexByCoord(vec2 pos, float tex_size) {
    float i = floor(pos.y - 0.5) * tex_size + floor(pos.x - 0.5);
    return floor(i * 0.5);
}

vec2 getCoordByIndex(float i, float tex_size, float itex_size) {
    i *= 2.0;
    return vec2(modfi2(i, tex_size), floor(i * itex_size)) * itex_size;
}

vec2 getCoordByIndexReproject(float i, float tex_size, float aspect) {
    i *= 2.0;
    vec2 asp_size = floor(vec2(tex_size * aspect, tex_size / aspect));
    return vec2(modfi2(i, asp_size.x), floor(i / asp_size.x)) / asp_size;
}

// [pos.xy, vel.xy]
bool isBufferA(vec2 pix_coord) {
    return mod(pix_coord.x, 2.0) < 1.0;
}

void readParticleDataByIndex(sampler2D tex, float i, float size, out vec4 A, out vec4 B) {
    float itex_size = 1.0 / size;
    vec2 p_coord = getCoordByIndex(i, size, itex_size);
    A = texture(tex, p_coord);
    B = texture(tex, p_coord + vec2(itex_size, 0.0));
}

vec4 encodeBufferB(vec3 color, float scale, float timeout) {
    return vec4(color.yy,scale,timeout);
}