attribute vec2 attrib0;
attribute vec2 attrib1;

uniform sampler2D tex0;

uniform mat4 transform;
uniform float time;
uniform vec4 size; // texture size, global scale, lifetime, timescale

varying vec4 coord0;
varying vec4 coord1;

#include <../common/math.glsl>
#include <common.glsl>

vec2 textureAtlas(vec2 uv, int i) {
    const vec2 SIZE = vec2(2.0,3.0);
    const vec2 CHUNK = vec2(1.0) / SIZE;
    vec2 start = vec2(float(i%2), float(i/3%3)) / SIZE;
    return uv * CHUNK + start;
}

void main() {
    vec4 buffer_a, buffer_b;
    float ind = float(gl_InstanceID);
    readParticleDataByIndex(tex0, float(gl_InstanceID), size.x, buffer_a, buffer_b);

    float t = max(buffer_b.x / size.z, 0.0);
    float scale = sqrt((1.0 - t));
    float alpha = t;
    
    // position
    vec2 ext = attrib0 * buffer_b.y * scale;
    ext = rotateZ(ext, buffer_b.z);
    vec2 vertex = (buffer_a.xy + ext) * size.y;

    gl_Position = transform * vec4(vertex.x,-vertex.y,0.0,1.0);
    coord0.xy = textureAtlas(attrib1,gl_InstanceID);
    coord0.z = alpha;

    // vertical falloff
    coord0.w = 1.0 + buffer_a.y * 10.0;
    coord1.x = 1.0 + vertex.y * 0.02;
}

[FS]
#ifdef GL_FRAGMENT_PRECISION_HIGH
    precision highp float;
#else
    precision mediump float;
#endif
precision lowp sampler2D;

uniform sampler2D tex0;
uniform sampler2D tex1;
uniform vec4 size;
uniform vec4 ct_mul;
uniform vec4 ct_add;

varying vec4 coord0;
varying vec4 coord1;

#if __VERSION__ >= 300
    layout(location = 0) out vec4 fragData_0;
#endif

void main() {
    float alpha = texture(tex1, coord0.xy).x;
    float vertical_falloff = clamp(coord0.w,0.0,1.0);
    vertical_falloff *= pow(clamp(coord1.x,0.0,1.0), 2.0);

    vec3 col = vec3(alpha * 0.5 + 0.5);
    gl_FragColor = vec4(col, coord0.z * alpha * vertical_falloff) * ct_mul + ct_add;
}
