attribute vec2 attrib0;
attribute vec2 attrib1;
varying vec2 coord0;

void main() {
    gl_Position = vec4(attrib0.x,attrib0.y,0.0,1.0);
    coord0 = attrib1;
}

[FS]
#ifdef GL_FRAGMENT_PRECISION_HIGH
    precision highp float;
#else
    precision mediump float;
#endif
precision lowp sampler2D;

uniform sampler2D tex0;
uniform vec4 time_ex;
uniform vec4 size; // texture size | global scale | lifetime | timescale
uniform vec4 params1; // spawning | vel cone dir | vel cone width | velocity

varying vec2 coord0;

/*
 * 8 floats per particle:
 * [pos, vel]
 * [time, scale, rot, -]
 */

#include <../common/math.glsl>
#include <common.glsl>

#if __VERSION__ >= 300
    layout(location = 0) out vec4 fragData_0;
#endif

vec2 getVelocity(vec2 rnd) {
    float phi = params1.y + rnd.x * params1.z;
    return vec2(sin(phi), cos(phi)) * rnd.y * params1.w;
}

void main() {
    float dt = time_ex.y;

    const float aspect = 1.0;
    const float scale = 1.0;
    float ind = getIndexByCoord(gl_FragCoord.xy, size.x);
    vec4 p = texture(tex0,coord0);
    vec2 coord = getCoordByIndexReproject(ind, size.x, aspect);
    coord = scaleAround(coord,vec2(0.5),vec2(scale));

    // buffer A
    if(isBufferA(gl_FragCoord.xy)) {   
        #ifdef INIT     
            vec2 rnd = rand21(ind * 0.01546124);
            p.xy = vec2(0.0); // pos
            p.zw = getVelocity(rnd); // vel
        #else            
            // semi-implicit Euler
            float ddt = dt * size.w;
    
            // integrate forces
            const vec2 GRAVITY = vec2(0.0,1.0);
            vec2 force = vec2(0.0);
            p.zw += (force + GRAVITY) * ddt;
            
            // integrate velocity
            p.xy += p.zw * ddt;

            // reset
            if(params1.x > 0.5) {
                float itex_size = 1.0 / size.x;
                vec4 B = texture(tex0, coord0 + vec2(itex_size, 0.0));
                if(B.x <= 0.0) {
                    vec2 rnd = rand21(ind * 0.01546124 + B.x);
                    p.xy = vec2(0.0); // pos
                    p.zw = getVelocity(rnd); // vel
                }
            }
        #endif

    // buffer B
    } else {
        vec2 rnd = rand21(ind * 0.018124);

        #ifdef INIT
            p = vec4(size.z * abs(rnd.y), // timeout
                     0.8, // scale
                     0.0, // angle
                     0.0);
        #else            
            float itex_size = 1.0 / size.x;
            vec4 A = texture(tex0, coord0 - vec2(itex_size, 0.0));
            p.z -= dt * A.z * rnd.x; // angle

            // reset
            if(params1.x > 0.5) {
                if(p.x <= 0.0) {
                    p.x = size.z;
                    p.z = 0.0; // angle
                }
            }
            p.x -= dt;
        #endif
    }

    gl_FragColor = p;
}
