attribute vec2 attrib0;
attribute vec2 attrib1;
attribute vec4 attrib2;
uniform mat4 uMVMatrix;
varying vec2 vTextureCoord;
varying vec4 vLight;

void main(void) {
    gl_Position = uMVMatrix * vec4(attrib0, 1.0, 1.0);
    vTextureCoord = attrib1;
    vLight = attrib2;
}

[FS]
precision mediump float;
#ifdef WITH_DEPTH
    uniform sampler2D tex0;
    uniform sampler2D tex1;
#else
    uniform sampler2D tex0;
#endif
uniform sampler2D tex2;
uniform vec3 airColor;
varying vec2 vTextureCoord;
varying vec4 vLight;
uniform vec2 offset;
uniform float focus;

uniform float visibilityK;
uniform float nearMeters;
uniform float farMeters;
uniform float horizonStartDepth;
uniform float horizonMeters;

#if __VERSION__ >= 300
    layout(location = 0) out vec4 fragData_0;
#endif

void main(void) {
    //high precision required to address pixels for parallax effect. Otherwise the animation is NOT smooth.
    #if defined(PARALLAX)
        #ifdef GL_FRAGMENT_PRECISION_HIGH//highp supported on device
            precision highp float;
        #endif
    #endif
    vec2 disCords = vTextureCoord;
    precision mediump float;

    #if defined(PARALLAX)
        float depth = 1. - texture(tex2, vTextureCoord).r; //[0..1]
        disCords.x += (depth - focus) * offset.x;
        disCords.y -= (depth - focus) * offset.y;
    #endif

    #ifdef WITH_DEPTH
        float distanceDepth = texture(tex1, disCords).r; //[0..1]
        float distanceLinear = nearMeters + distanceDepth / horizonStartDepth * (farMeters - nearMeters);
        float distanceCurved = max(distanceDepth - horizonStartDepth, 0.0) / (1.001 - horizonStartDepth);
        distanceCurved = distanceCurved * distanceCurved * distanceCurved * (horizonMeters - farMeters);
        float distanceMeters = (distanceLinear + distanceCurved) * PRECISION_ADJUSTMENT_K;

        float airAlpha = 0.;
        //@see AirModel.findDistanceAirCover()
        airAlpha = (1. - exp( -visibilityK * distanceMeters)) * vLight.a;

        vec4 result = texture(tex0, disCords) * vLight; 
        result.rgb = result.rgb * (1. - airAlpha) + airAlpha * airColor.rgb; 
        gl_FragColor = result;
        //    gl_FragColor = vec4(1., ) * texture2D(depthTexture, vTextureCoord).r;
    #else
        vec4 textureColor = texture(tex0, disCords);
        vec4 color = textureColor * vLight;

        //Reason: outline the edge of landscape with horizon color.
        //This makes AI generated masks look more natural on sunset.
        //See https://youtu.be/-YB1NcBZWDY
        //        float kEdge = textureColor.w;
        //        color.rgb = color.rgb * kEdge + airColor.rgb * (1. - kEdge);
        gl_FragColor = color;
    #endif
}