attribute vec2 attrib0;
attribute vec2 attrib1;
varying vec2 coord0;

void main() {
    gl_Position = vec4(attrib0, 1.0, 1.0);
    coord0 = attrib1;
}

[FS]
precision mediump float;

uniform sampler2D tex0; // frame 1
uniform sampler2D tex1; // frame 2
uniform float geo_frame_time;
uniform vec4 offset;
uniform vec4 params3; // lengthy reflections, smooth, LF wave height, LF wave freq

varying vec2 coord0;

#if __VERSION__ >= 300
    layout(location = 0) out vec4 fragData_0;
#endif

vec3 combineNormals1(vec3 n0, vec3 n1) {
    return vec3(n0.xy + n1.xy, n0.z * n1.z);
}

vec4 getLayer(float freq) {
    float freq_scaled = freq * 2.0;
    vec2 uv = coord0 * freq_scaled + offset.zw;
    vec4 data = mix(texture(tex0,uv), texture(tex1,uv), geo_frame_time) * 2.0 - 1.0;
    // data.xy *= freq;

    float freq_main = params3.w * freq;
    const float UNPACK_SCALE = 10.0;
    vec3 n = vec3(data.xy, sqrt(abs(1.0 - dot(data.xy,data.xy))));
    n.xy *= freq_main * params3.z * UNPACK_SCALE;
    n.x = -n.x;
    n = normalize(n);

    return vec4(n, data.z);
}

void main() {
    vec4 data0 = getLayer(1.0);
    vec4 data1 = getLayer(0.5);

    // normal
    data0.xyz = combineNormals1(data0.xyz, data1.xyz);
    // data0.xyz = normalize(data0.xyz);

    // height
    data0.w += data1.w;

    gl_FragColor = data0 * 0.5 + 0.5;
}

