attribute vec2 attrib0;
attribute vec2 attrib1;
uniform lowp vec4 params;
uniform lowp vec4 params2;
varying vec2 coord0;
varying lowp vec2 proj;

const float PI = 3.141592;

void main() {
    coord0 = attrib1;
    float ang_x_rad = params.y / 180.0 * PI;
    proj.x = params2.x * tan(-ang_x_rad);

    gl_Position = vec4(attrib0, 1.0, 1.0);
}

[FS]
#ifdef GL_FRAGMENT_PRECISION_HIGH
    precision highp float;
#else
    precision mediump float;
#endif
precision lowp sampler2D;

uniform lowp sampler2D tex0; // bg
uniform sampler2D tex1; // mask
uniform sampler2D tex2; // ripple
uniform sampler2D tex3; // cracks
uniform sampler2D tex4; // noise

uniform lowp vec4 params; // camY, angX, wind speed norm, biasY
uniform lowp vec4 params2; // dirZ, fresnelF0, rippleHeight, reflectivity
uniform lowp vec4 params3; // lengthy reflections, smooth, LF wave height, LF wave freq
uniform lowp vec4 params4; // LF wave choppiness, reflection percent
uniform lowp vec4 params5; // foam decay, foam threshold
uniform lowp vec4 color;
uniform vec4 offset;
uniform lowp vec4 fog_params; // color, mistK * distance scale
uniform lowp vec4 fog_params2; // units2meters, 
uniform lowp vec4 sun_params;
uniform lowp vec4 sun_color;
uniform lowp vec4 ambient;
uniform lowp mat2 rotation;
uniform lowp vec4 touch;
uniform lowp vec2 uv_ratio;
uniform vec4 resolution;

varying vec2 coord0;
varying lowp vec2 proj;

#if __VERSION__ >= 300
    layout(location = 0) out vec4 fragData_0;
#endif

#define FLIP_Y
#define WIND_SPEED params.z

const float PI = 3.141592;
const float HORIZONTAL_STRETCH = 0.5;
const float ICE_FREQ = 1.0;
const float NIGHT_CRACKS = 0.1;
const float SNOW_ALPHA = 0.8;

vec3 toLinear(vec3 c) { return c * c; }
vec3 toSRGB(vec3 c) { return sqrt(c); }

vec2 scaleAround(vec2 x, vec2 p, float s) { return (x - p) * s + p; }

float intersectPlaneRay(vec3 plane, vec2 ori, vec2 dir) {
    return (dot(plane.xy,ori)+plane.z) / dot(plane.xy,-dir);
}

vec3 combineNormals1(vec3 n0, vec3 n1) {
    return vec3(n0.xy + n1.xy, n0.z * n1.z);
}

float getSunLight() {
    mediump float sun_light = distance(coord0.x * resolution.x, sun_params.x);
    sun_light = smoothstep(fog_params2.y*3.0+sun_params.z*0.5, fog_params2.y-sun_params.z*0.7, sun_light); // section length instead of radius
    sun_light *= sun_light;
    
    lowp float vertical_falloff = max(1.0 - distance(coord0.y * resolution.y, fog_params2.z) / (sun_params.z * 0.5), 0.0);
    sun_light *= vertical_falloff * vertical_falloff;

    mediump float horizon_proximity = distance(coord0.y * resolution.y, sun_params.y);
    horizon_proximity = smoothstep(sun_params.z*1.5, sun_params.z * 0.5, horizon_proximity);

    return sun_light * horizon_proximity;
}

lowp float getFogDensity(float distanceUnits, mediump float sun_light) {
    // distanceUnits = min(distanceUnits, 250.0);
    lowp float distanceM = max(distanceUnits * fog_params2.x, 0.0);
    return max(1.0 - exp(-fog_params.w * distanceM), 0.0) * (1.0 - sun_light * fog_params2.w * sun_color.w);
}

vec3 getReflection(vec2 uv, float lod) {
    lowp vec3 ret = texture(tex0, uv, lod).xyz;
    return ret;
}

vec2 toMaskSpace(vec2 uv) {
    uv = scaleAround(uv,vec2(0.5,0.0),touch.z);
    uv += touch.xy;
    uv.y /= uv_ratio.y;
    uv = uv * 2.0 - 1.0;
#ifdef FLIP_Y
    uv.y = -uv.y;
#endif
    uv.x *= resolution.z * HORIZONTAL_STRETCH; 
    return uv;
}

vec2 toImageSpace(vec2 uv) {
    uv.x *= resolution.w / HORIZONTAL_STRETCH; 
#ifdef FLIP_Y
    uv.y = -uv.y;
#endif
    uv = uv * 0.5 + 0.5;
    uv.y *= uv_ratio.y;
    uv -= touch.xy;
    uv = scaleAround(uv,vec2(0.5,0.0), 1.0/touch.z);
    return uv;
}

vec3 getSunDirection() {
    vec2 mask_uv = toMaskSpace(sun_params.xy / resolution.xy);
    mask_uv.y = mask_uv.y * 0.5 + 0.5;
    vec3 dir = vec3(mask_uv, params2.x); 
    return normalize(dir);
}

float blinn(vec3 light, vec3 eye, vec3 normal, float shiness){
	float nrm = (shiness + 8.0) / (3.1415 * 8.0);
	vec3 _half = normalize(light + eye);
	return pow(max(dot(_half,normal),0.0),shiness) * nrm;
}

// main
void main() {
    lowp vec2 uv = toMaskSpace(coord0);
#ifndef NO_MASK
    vec2 mask_uv = uv;
    mask_uv.x *= resolution.w / HORIZONTAL_STRETCH; 
    lowp float mask_tex = texture(tex1, mask_uv*0.5+0.5).x;
    if(mask_tex < 0.001) discard;
#endif  

    // camera
    lowp vec3 dir = vec3(uv, params2.x);
    dir.yz *= rotation;
    mediump vec3 ori = vec3(0.0,params.x,3.0);
    mediump vec3 ori_choppy = ori;
    dir = normalize(dir);  

    // flat reflection plane
    lowp float yproj = params2.x * tan(-params.y / 180.0 * PI);
    lowp vec3 refl_plane = vec3(0.0,1.0,-yproj);        
    lowp float dist = dot(refl_plane.xy,uv) + refl_plane.z;
    lowp float wave_height = 1.0;

    // surface
    float p_dist = ori_choppy.y / dir.y;
    vec3 p = ori_choppy - dir * p_dist;
    
    // reflection normal
    vec2 rns = vec2(params2.z,1.0);
    vec2 nuv = p.xz * params4.zw;
    mediump vec3 n = normalize((texture(tex2, nuv).xyz * 2.0 - 1.0) * rns.xxy);
    n.y = -n.y;  

    // normal in screen space
    lowp vec2 nss = normalize(n.xz * vec2(0.25,1.0));
    
    // reflection
    // reflection
    dist = (uv.y - proj.x) / nss.y; // intersectPlaneRay
    lowp float lod = log2(1.0+dist*params3.y); // blur stars & avoid derivatives pixellation tradeoff
    lowp vec2 refl_offset = nss * (uv.y - proj.x) * 2.0;

    // flip along horizon line
    lowp float refl_ray_scale = max(params3.x * (n.y + 1.0), 0.5);
    lowp float horizon_offset = refl_offset.y * 0.5;
    refl_offset *= refl_ray_scale;
    refl_offset.y -= (refl_offset.y - horizon_offset) * 2.0;

    lowp vec2 ruv = uv - refl_offset;
    lowp vec2 ruv_is = toImageSpace(ruv);

    // dist = -intersectPlaneRay(refl_plane, uv, nss);
    // lowp float lod = log2(1.0+dist*params3.y);
    // lod += smoothstep(0.4,0.7, texture(tex4,p.xz * vec2(0.05,0.2)).x) * 1.0;
    // lowp vec2 ruv = uv - nss * (dist - params.w) * max(2.0 * params3.x * (n.y + 1.0), 1.0);
    // lowp vec2 ruv_is = toImageSpace(ruv);
    lowp vec3 reflected = getReflection(ruv_is, lod);
    reflected *= reflected;

    // refraction
    lowp vec3 refracted = toLinear(color.xyz);

#ifdef USE_REAL_REFLECTION
    lowp vec2 rfr_uv = uv - (nss) * params3.x * n.y * 0.5;
    lowp vec3 refracted_image = texture(tex0, toImageSpace(rfr_uv), lod * 0.7).xyz;
    refracted_image = toLinear(refracted_image);
    reflected = mix(refracted_image, reflected, params4.y);
    refracted = mix(refracted_image, reflected, min(params4.y*4.0,1.0));
#endif   

    // cracks
    float cracks = texture(tex3,p.xz * 0.25).y; // base
    cracks += texture(tex3,p.xz * 0.8).y * 0.4; // high freq
    cracks += texture(tex3,p.xz * 0.15, 3.0).y * 0.75;
    cracks += texture(tex3,p.xz * 0.05, 4.0).y * 0.5;
    vec3 deep_ice_color = mix(vec3(0.0,0.35,0.5), toLinear(fog_params.xyz) * 0.1, 0.9);
    refracted *= mix(deep_ice_color, vec3(1.0), cracks * 0.7 + 0.3);

    float night_level = 1.0 - max(ambient.y * 1.5 - 0.5, 0.0);
    refracted += cracks * night_level * NIGHT_CRACKS * 0.65; // brighter in the night
    
    float sun_light = getSunLight();
    float fog_density = getFogDensity(distance(p, ori), sun_light);
    float y_alpha = clamp(smoothstep(-1.0,-0.99,uv.y), 0.0,1.0);

    float zdist = ori.z - p.z;
    vec2 flat_p = ori.xz - dir.xz * (ori.y / dir.y);
    vec2 foam_uv = mix(p.xz, flat_p, params5.w);
    float hnoise = texture(tex4,p.xz * 7.0).x;
    vec3 L = getSunDirection() * vec3(1.,1.,1.);

    // add some light to reflection
    // reflected += blinn(-L, dir, n.xzy, 30.0) * sun_params.w * 0.4 * hnoise * hnoise * hnoise;

    // mix
    vec3 fwd_dir = normalize(vec3(0.0, dir.yz));
    float fade = smoothstep(0.7,0.5,abs(ruv_is.y-0.5));
    float fresnel = clamp(pow(abs(dot(vec3(0.0,1.0,0.0),fwd_dir)),1.0/3.0), 0.0,1.0);
    fresnel = params2.y + fresnel * (1.0 - params2.y) * params2.w;
    fresnel = 1.0 - (1.0 - fresnel) * fade;
    fresnel *= y_alpha;
    lowp vec3 wcol = mix(reflected, refracted, fresnel);
    wcol = sqrt(wcol);

    // snow
    float snow = texture(tex4,p.xz * 0.08).x;
    const float SNOW_LEVEL = 0.7;
    snow = smoothstep(SNOW_LEVEL,SNOW_LEVEL+0.2,snow);

    float total_snow = snow;

#ifdef SNOW
    float flying_snow = texture(tex4,p.xz * 0.07 + offset.xy).x;
    flying_snow *= flying_snow * sqrt(params.z);
    total_snow += flying_snow * 0.55;
#endif

    wcol = mix(wcol, ambient.xyz, total_snow * SNOW_ALPHA);
    wcol += total_snow * night_level * NIGHT_CRACKS; // brighter in the night

    // snow light        
    wcol += blinn(-L, dir, n.xzy, 40.0) * sun_params.w * snow * 0.5 * hnoise * hnoise * hnoise;

    // fog
    wcol = mix(wcol, fog_params.xyz, fog_density);

    // debug
    // wcol = vec3(fog_density);
    // wcol = n * 0.5 + 0.5;
    // wcol = texture(tex5,p.xz).xyz;
    // if(coord0.x > 0.5) wcol = texture(tex5,coord0*2.0).xyz;
 
    // combine
    lowp float mask = step(0.0,dir.y) * color.w;
#ifndef NO_MASK
    mask *= mask_tex;
#endif

    gl_FragColor = vec4(wcol, mask);
}
