// https://www.gdcvault.com/play/203/Fast-Water-Simulation-for-Games

attribute vec2 attrib0;
attribute vec2 attrib1;
uniform vec4 dt;
varying vec2 coord0;
varying float damp;

void main() {
    gl_Position = vec4(attrib0, 1.0, 1.0);
    coord0 = attrib1;

    // damp = 1.0 - (1.0 - dt.z) * dt.y;
    damp = exp(-dt.x * (1.0 - dt.z));
}

[FS]
#ifdef GL_FRAGMENT_PRECISION_HIGH
    precision highp float;
#else
    precision mediump float;
#endif

uniform sampler2D tex0; // prev data
uniform vec4 params0;
uniform vec4 touch0;
uniform vec4 touch1;
uniform vec4 dt;
uniform bool is_init;

varying vec2 coord0;
varying float damp;

#if __VERSION__ >= 300
    layout(location = 0) out vec4 fragData_0;
#endif

const float RAD = 2.0;
    
const float HEIGHT_SPRING = 1.0;
const float HEIGHT_FLUID = 1.0;
const float HEIGHT_RAIN = 1.1;
const float SPRING_FREQ = 1.92;
const float NUM_CELLS = 80.0;
const float MAX_SLOPE = 0.1;

float line(vec2 p, vec2 p0, vec2 p1) {
    vec2 d = p1 - p0;
    float t = clamp(dot(d,p-p0) / dot(d,d), 0.0,1.0);
    vec2 proj = p0 + d * t;
    return length(p - proj);
}

float hash13(vec3 p3) {
	p3  = fract(p3 * .1031);
    p3 += dot(p3, p3.yzx + 33.33);
    return fract((p3.x + p3.y) * p3.z);
}

vec2 hash22(vec2 p) {
	vec3 p3 = fract(vec3(p.xyx) * vec3(.1031, .1030, .0973));
    p3 += dot(p3, p3.yzx+33.33);
    return fract((p3.xx+p3.yz)*p3.zy);
}

vec4 update(vec2 uv, vec2 pix) {
    vec4 d = texture(tex0, uv);
    
    // velocity
    float avg = (texture(tex0, uv + pix * vec2(-1, 0)).x +
              texture(tex0, uv + pix * vec2( 1, 0)).x +
              texture(tex0, uv + pix * vec2( 0,-1)).x +
              texture(tex0, uv + pix * vec2( 0, 1)).x) * 0.25;
    float f = (avg - d.x);
    
    f *= 0.6; // decrease offset for stability reasons
    f = clamp(f, -MAX_SLOPE, MAX_SLOPE); // clamp offset for stability reasons

    d.y += f * dt.x;
    d.y *= damp * 0.99;
    d.x += d.y * dt.x;
    d.x = mix(avg,d.x,damp*0.98); // blur
    
    return d;
}

void main() {
    if(is_init) {
        gl_FragColor = vec4(0.0);

    } else {
        vec2 uv = coord0;
        vec2 pix = 1.0 / params0.xy;
        
        float w = 0.0;
        vec4 d = update(uv, pix); w += 1.0;
        d += update(uv, pix*2.0) * 0.5; w += 0.5;
        //d += update(uv, pix*4.0) * 0.25; w += 0.25;
        //d += update(uv, pix*8.0) * 0.1; w += 0.1;
        d /= w;    

        // touch
#ifdef TOUCH
        float ld = max(line(gl_FragCoord.xy, touch0.xy, touch0.zw) - RAD, 0.0);
        ld = max(2.0 - ld, 0.0) / 2.0;
        d.x += ld * ld * HEIGHT_FLUID;

        ld = max(line(gl_FragCoord.xy, touch1.xy, touch1.zw) - RAD, 0.0);
        ld = max(2.0 - ld, 0.0) / 2.0;
        d.x += ld * ld * HEIGHT_FLUID;
#endif        
        // rain
#ifdef RAIN
        const float NUM_CELLS = 50.0;
        vec2 cid = uv * NUM_CELLS;
        vec2 cuv = fract(cid);
        cid = floor(cid);
        vec2 offset = hash22(cid) * 2.0 - 1.0;
        float drop_dist = max(1.0 - length(cuv - 0.5 + offset * 0.2) * 0.5, 0.0);
        //if (iFrame == 0) {
            
            if(hash13(vec3(cid,fract(params0.z))) > 0.9999 - params0.w * 0.002) {   
                d.x += smoothstep(0.0,0.9,drop_dist) * HEIGHT_FLUID * HEIGHT_RAIN;
            }
            //d = vec4(0.5);
        //}            
#endif

        gl_FragColor = d;
    }
}