// https://www.gdcvault.com/play/203/Fast-Water-Simulation-for-Games

attribute vec2 attrib0;
attribute vec2 attrib1;
varying vec2 coord0;

void main() {
    gl_Position = vec4(attrib0, 1.0, 1.0);
    coord0 = attrib1;
}

[FS]
precision mediump float;

uniform sampler2D tex0; // data
uniform vec2 resolution;
uniform float time;

varying vec2 coord0;

#if __VERSION__ >= 300
    layout(location = 0) out vec4 fragData_0;
#endif

const float RAD = 5.0;
    
const float HEIGHT_SPRING = 1.0;
const float HEIGHT_FLUID = 0.2;
const float HEIGHT_RAIN = 0.3;
const float SPRING_FREQ = 1.92;
const float NUM_CELLS = 50.0;

float hash13(vec3 p3) {
	p3  = fract(p3 * .1031);
    p3 += dot(p3, p3.yzx + 33.33);
    return fract((p3.x + p3.y) * p3.z);
}

vec2 hash22(vec2 p) {
	vec3 p3 = fract(vec3(p.xyx) * vec3(.1031, .1030, .0973));
    p3 += dot(p3, p3.yzx+33.33);
    return fract((p3.xx+p3.yz)*p3.zy);
}

float react( in float p, in vec2 uv, in vec2 pix, in vec2 qid, float rl ) {
    vec4 q = texture(tex0, uv + pix * qid);
    
    float di = q.x - p;
    float l = length(qid);
    
    return 0.05*di/l;
}

vec4 update(vec2 uv, vec2 pix) {
    vec4 p = texture(tex0, uv);
    
    // acceleration
    //p.x *= 0.999;
    
    // constrains
    float npp = p.x;
    npp += react( p.x, uv, pix, vec2(-1, 0), 0.0 );
    npp += react( p.x, uv, pix, vec2( 1, 0), 0.0 );
    npp += react( p.x, uv, pix, vec2( 0,-1), 0.0 );
    npp += react( p.x, uv, pix, vec2( 0, 1), 0.0 );
    
    npp += react( p.x, uv, pix, vec2(-1,-1), 0.0 );
    npp += react( p.x, uv, pix, vec2(-1, 1), 0.0 );
    npp += react( p.x, uv, pix, vec2( 1,-1), 0.0 );
    npp += react( p.x, uv, pix, vec2( 1, 1), 0.0 );
    
    p.x = npp;
    
    // innertia
    float np = SPRING_FREQ * p.x - p.y;
    p.y = p.x;
    p.x = np;
    
    return p;
}

void main() {
    vec2 uv = coord0;
    vec2 pix = 1.0 / resolution.xy;
    
    float w = 0.0;
    vec4 d = update(uv, pix); w += 1.0;
    d += update(uv, pix*2.0) * 0.5; w += 0.5;
    d += update(uv, pix*4.0) * 0.25; w += 0.25;
    //d += update(uv, pix*8.0) * 0.1; w += 0.1;
    d /= w;
        
    // rain
    vec2 cid = uv * NUM_CELLS;
    vec2 cuv = fract(cid);
    cid = floor(cid);
    vec2 offset = hash22(cid) * 2.0 - 1.0;
    float drop_dist = max(1.0 - length(cuv - 0.5 + offset * 0.2) * 2.0, 0.0);
    //if (iFrame == 0) {
        if(hash13(vec3(cid,fract(time))) > 0.999) {    
            d.x += smoothstep(0.0,0.9,drop_dist) * HEIGHT_SPRING * HEIGHT_RAIN;// * 0.2;
        }
        //d = vec4(0.5);
    //}  
    // d.x = sin(uv.x*10.0) * sin(uv.y*10.0);
    // d.x += 0.5;
    // d.xy = uv;
    // d.x = hash13(vec3(cid,1.0));
    d.w = 1.0;
    
    gl_FragColor = d;
}